#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>

void help(void)
{
	fprintf(stderr,"info </dev/comediN>\n");
	exit(0);
}

char *tobinary(char *s,int bits,int n);


int main(int argc,char *argv[])
{
	int i;
	comedi_t *it;
	comedi_subdinfo *s;
	
	if(argc!=2)help();

	it=comedi_open(argv[1]);
	if(!it){
		fprintf(stderr,"cannot open %s\n",argv[1]);
		exit(0);
	}

	printf("overall info:\n");
	printf("  version code: 0x%06x\n",it->devinfo.version_code);
	printf("  driver name: %s\n",it->devinfo.driver_name);
	printf("  board name: %s\n",it->devinfo.board_name);
	printf("  number of subdevices: %d\n",it->n_subdevices);
	
	for(i=0;i<it->n_subdevices;i++){
		s=it->subdevices+i;

		printf("subdevice %d:\n",i);
		printf("  type: %d (%s)\n",s->type,"unknown");
		printf("  number of channels: %d\n",s->n_chan);
		printf("  max data value: %d\n",s->maxdata);
	}
#if 0
	for(i=0;i<info.chan;i++){
		printf("channel %d:\n",i);
		j=0;
		for(;;){
			param.chan=i;
			param.pnum=j;
			ret=ioctl(it,COMEDI_GETNPARAM,&param);
			if(ret<0)break;
			
			printf("   %-4.4s ",flagstring[param.flags]);
			switch(param.pnum){
			case COMEDI_IO:
				switch(param.pval){
				case COMEDI_IO_INPUT:
					printf("input");
					break;
				case COMEDI_IO_OUTPUT:
					printf("output");
					break;
				case COMEDI_IO_BIDIR:
					printf("bidirectional");
					break;
				default:
					printf("input/output unknown");
					break;
				}
				break;
			case COMEDI_AD:
				switch(param.pval){
				case COMEDI_AD_ANALOG:
					printf("analog");
					break;
				case COMEDI_AD_DIGITAL:
					printf("digital");
					break;
				default:
					printf("analog/digital unknown");
					break;
				}
				break;
			case COMEDI_BITS:
				printf("number of bits=%d",param.pval);
				break;
			case COMEDI_MAXDATA:
				printf("max data=%d",param.pval);
				break;
			case COMEDI_RANGE:
				printf("range=[%d]",param.pval);
				break;
			case COMEDI_LASTDATA:
				printf("last data=%d",param.pval);
				break;
			case COMEDI_LASTSEC:
				printf("last sec=%d",param.pval);
				break;
			case COMEDI_LASTNSEC:
				printf("last nsec=%d",param.pval);
				break;
			case COMEDI_AREF:
				printf("analog reference: ");
				switch(param.pval){
				case COMEDI_AREF_GND:
					printf("ground");
					break;
				case COMEDI_AREF_COM:
					printf("floating common");
					break;
				case COMEDI_AREF_DIFF:
					printf("differential");
					break;
				case COMEDI_AREF_OTHER:
					printf("other");
					break;
				default:
					printf("unknown");
					break;
				}
				break;
			case COMEDI_DITHER:
				printf(param.pval?"dither enabled"
					:"dither disabled");
				break;
			case COMEDI_DEGLITCH:
				printf(param.pval?"deglitch enabled"
					:"deglitch disabled");
				break;
			case COMEDI_STROBE:
				printf(param.pval?"strobe enabled"
					:"strobe disabled");
				break;
			case COMEDI_IOBITGROUP:
				printf("# of bits grouped for I/O determination: %d",
					param.pval);
				break;
			case COMEDI_IOBITS:
				printf("I/O bitfield (1=output): 0b%s",
					tobinary(s,param.pval,16));
				break;
			case COMEDI_COUNT:
				printf("counter value: %d\n",param.pval);
				break;
                        case COMEDI_RANGE_TYPE: {
                          comedi_range_info *range_info;
                          int k;
                          printf("ranges: ");
                          comedi_get_range_info(it, i, &range_info);
                          for (k = 0 ; k < range_info->n_ranges ; k++) {
                            printf("%s{%g:%g}",
                                   k?", ":"",
                                   range_info->range[k].min,
                                   range_info->range[k].max);
                          }
                        } break;
			default:
				printf("unknown %d",param.pval);
			}
			printf("\n");
			j++;
		}
	}
#endif
	
	return 0;
}

char *tobinary(char *s,int bits,int n)
{
	int bit=1<<n;
	char *t=s;
	
	for(;bit;bit>>=1)
		*t++=(bits&bit)?'1':'0';
	*t=0;
	
	return s;
}

