/*
    module/8255.c
    Driver for 8255

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
   This file contains an exported subdevice for driving an 8255.

 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>



#define _8255_SIZE 4

#define _8255_DATA 0
#define _8255_CR 3


static int subdev_8255_cb(int dir,int port,int data,void *arg)
{
	int iobase=(int)arg;

	if(dir){
		outb(iobase+port,data);
		return 0;
	}else{
		return inb(iobase+port);
	}
}

int subdev_8255_dio(comedi_device *dev,comedi_subdevice *s,comedi_trig *it)
{
	int chan;
	int iobase=(int)s->range_list;

	chan=CR_CHAN(it->chanlist[0]);

	it->data[0]=inb(iobase+_8255_DATA+chan);

	return 1;
#if 0
	outb(it->data[0],iobase+_8255_DATA+chan);
#endif
}

#if 0
static int subdev_8255_sp(comedi_device *dev,comedi_param *it)
{
	int cr=devpriv->cr;
	
	switch(it->pnum){
	case COMEDI_IOBITS:
		switch(it->chan){
		case 0:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x10;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x10;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 1:
			switch(it->pval){
			case 0xff&COMEDI_IOBITS_INPUT:
				cr |= 0x02;
				break;
			case 0xff&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x02;
				break;
			default:
				return -EINVAL;
			}
			break;
		case 2:
			switch(0x0f&it->pval){
			case 0x0f&COMEDI_IOBITS_INPUT:
				cr |= 0x01;
				break;
			case 0x0f&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x01;
				break;
			default:
				return -EINVAL;
			}
			switch(0xf0&it->pval){
			case 0xf0&COMEDI_IOBITS_INPUT:
				cr |= 0x08;
				break;
			case 0xf0&COMEDI_IOBITS_OUTPUT:
				cr &= ~0x08;
				break;
			default:
				return -EINVAL;
			}
			break;
		default:
			return -EINVAL;
		}
		changeparam(&dev->chinfo[it->chan].paramlist,
			COMEDI_IOBITS,0,it->pval);
		outb(0x80|cr,iobase+_8255_CR);
		devpriv->cr=cr;
		return 0;
	default:
		return -EINVAL;
	}
}
#endif


int subdev_8255_init(comedi_device *dev,comedi_subdevice *s,int (*cb)(int,int,int,void *),void *arg)
{
	s->type=COMEDI_SUBD_DIO;
	s->n_chan=24;
	s->range_type=RANGE_digital;
	s->maxdata=1;

	/* commandeer range_list */
	s->range_list=arg;

	/* same for flaglist */
	if(cb==NULL){
		s->flaglist=(void *)subdev_8255_cb;
	}else{
		s->flaglist=(void *)cb;
	}

	return 0;
}


/*

   Start of the 8255 standalone device

 */

comedi_devinit dev_8255_init;
static int dev_8255_rem(comedi_device *dev);

int dev_8255_init(comedi_device *dev,comedi_devconfig *it)
{
	int ret;

	if(strcmp("8255",it->board_name))
		return 0;

	dev->iobase=it->options[0];
	printk("comedi%d: 8255: 0x%04x ",dev->minor,dev->iobase);
	if(check_region(dev->iobase,_8255_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(dev->iobase,_8255_SIZE,"8255");
	dev->iosize=_8255_SIZE;
	dev->driver_name="8255";
	dev->board_name="8255";

	dev->n_subdevices=1;

	if((ret=alloc_subdevices(dev))<0)
		return ret;

	subdev_8255_init(dev,dev->subdevices+0,NULL,(void *)dev->iobase);

#if 0
	devpriv->cr=0x1b;
	outb(devpriv->cr,dev->iobase+_8255_CR);
#endif

	dev->rem=dev_8255_rem;

	printk("\n");
	return 1;
}

static int dev_8255_rem(comedi_device *dev)
{
	printk("comedi%d: 8255: remove\n",dev->minor);
	
	kfree(dev->subdevices);
	release_region(dev->iobase,dev->iosize);

	return 0;
}

